import {
  JsonController,
  Param,
  Body,
  Get,
  Post,
  Put,
  Delete,
} from "routing-controllers";
import { getConnectionManager, Repository } from "typeorm";
import { EntityFromBody } from "typeorm-routing-controllers-extensions";
import { ResponseSchema } from "routing-controllers-openapi";
import { Track } from "../models/Track";
import { IsInt, IsNotEmpty, IsPositive, IsString } from "class-validator";

class CreateTrack {
  @IsString()
  @IsNotEmpty()
  name: string;

  @IsInt()
  @IsPositive()
  length: string;
}

@JsonController("/track")
export class TrackController {
  private trackRepository: Repository<Track>;

  constructor() {
    this.trackRepository = getConnectionManager().get().getRepository(Track);
  }

  @Get()
  @ResponseSchema(Track, { isArray: true })
  getAll() {
    return this.trackRepository.find();
  }

  @Get("/:id")
  @ResponseSchema(Track)
  getOne(@Param("id") id: number) {
    return this.trackRepository.findOne({ id: id });
  }

  @Post()
  post(@Body({ validate: true }) track: CreateTrack) {
    return this.trackRepository.save(track);
  }

  @Put("/:id")
  put(@Param("id") id: number, @EntityFromBody() track: Track) {
    return this.trackRepository.update({ id: id }, track);
  }

  @Delete("/:id")
  remove(@Param("id") id: number) {
    return this.trackRepository.delete({ id: id });
  }
}
